// Game Hero Midi Player

// thunderstruck
// March 4, 2011

// This example program shows how to use the GameHero Midi Player.
// Controls: A/B/L/R: play midis
//
// The midi player works by setting up a library of known midi files which can 
// be played during the game. Midis can be converted to a VB Header format using
// the Midi2VB Converter. The output should be added to ths "music" folder and integrtated
// into the midiLibrary.c 
//
// After each video frame, it checks each sound to see whether it should switch to
// a different note, stop the sound, or start playing it from the beginning.
//
// The following functions are provided:
//
// * ml_iniLibrary()
//   This must be called at the start of the game to initialize the Midi Library.
//
// * mp_init()
//   This must be called at the start of the game to initialize the Midi-Player.
//
// * mp_playMusic()
//   This must be called after each video frame for actually playing sounds.
//   The return value is the number of current note played. Pauses are counted as notes.
//
// * mp_playerStop()
//   Stops the music playback
//
// * mp_playerStart()
//   Starts the music playback
//
// * mp_loadSong(int index)
//   Loads a song from the midi library. 
//
// The engine is very basic, but can be expanded
//
// You may use the sound engine in your projects as long as you properly give
// credit.
//
// The Midi-Player is pretty close to what HorvatM did with his Soviet Union 2011 sound engine,
// even though our projects are totally unrelated. Actually, our projects are that similar, that
// I was able to copy most of this text from his project.
// However, there are major differences. It basically comes down to this:
// - If you want to have Sounds in your game you should go with the Soviet Union 2011 sound engine
// - If you want to play music in the backround you may want to use the GameHero Midi Player
//
// The Midi2VB converter supports both projects, so the choice is yours.


#include "./libgccvb/libgccvb.h"

#include "fonts/fontChar.h"
#include "imgs/logoChar.h"
#include "imgs/logoMap.h"

#include "player/midiPlayer.h"

 
int main()
{
	u8 start=1;
	int currNoteDebug = 0;
	int lastNoteDebug = -1;
	HWORD pad;
	
	vbDisplayOn();
	copymem((void*)CharSeg0, (void*)LOGOCHAR, 333*16); 
	copymem((void*)BGMap(0), (void*)LOGOMAP, 8192);
	copymem ((void*)CharSeg3, (void*)FONT, 512*16);
	
	//inti MidiLibrary
	ml_iniLibrary();
	
	//init MidiPlayer
	mp_init();
	
	while(1)
	{
		//playMusic (call this every frame)
		currNoteDebug = mp_playMusic();
		
		//setting up all images
		if(start==1)
		{
			WORLD_HEAD(31,WRLD_ON | WRLD_BGMAP | 0);
			WORLD_GSET(31,78,0,5);
			WORLD_SIZE(31,228,192);
			
			
			WORLD_HEAD(30,WRLD_ON | WRLD_BGMAP | 1);
			WORLD_GSET(30,148,-10,192);
			WORLD_SIZE(30,96,32);
			
			WORLD_HEAD(29, WRLD_END);
			start=0;
			vbDisplayShow();
		}
		
		//print out the "played note count"
		if(currNoteDebug != lastNoteDebug)
		{
			vbTextOutS(1, 1, 1, itoa(currNoteDebug, 10, 1));
			lastNoteDebug = currNoteDebug;
		}
		
		//check input
		if(pad&K_A) //metroid
		{
			mp_playerStop(); 	// stop player
			mp_loadSong(0);		// load song
			mp_playerStart();	// start loaded song
		}
		else if(pad&K_B) //zelda
		{
			mp_playerStop();
			mp_loadSong(1);
			mp_playerStart();
		}
		else if(pad&K_LT) // smb1
		{
			mp_playerStop();
			mp_loadSong(2);
			mp_playerStart();
		}
		else if(pad&K_RT) // smb cave
		{
			mp_playerStop();
			mp_loadSong(3);
			mp_playerStart();
		}
		
		pad = vbReadPad(); 	//get buttons
		vbWaitFrame(0);		//wait 
	}
    
    return 0;
}
