
int mp_songTick = 0;
int mp_currNote = 0;
int mp_targetTick = 0;
u16 mp_play = 0;

u8 *mp_timing;
int *mp_channel1;
int *mp_channel2;
int *mp_channel3;
int *mp_channel4;
int *mp_channel5;
u8 mp_channel1Active;
u8 mp_channel2Active;
u8 mp_channel3Active;
u8 mp_channel4Active;
u8 mp_channel5Active;
u16 mp_noteCount;
int mp_replay;

void mp_init()
{
	copymem((void*)WAVEDATA1, (void*)PIANO, 128);
}

//sets how many notes should be played
void mp_setNoteCount(u16 mp_noteCountI)
{
	mp_noteCount=mp_noteCountI;
}

//sets the replay
void mp_setReplay(u8 replayI)
{
	mp_replay=replayI;
}

//sets the mp_timing
void mp_setTiming(u8 *mp_timingI)
{
	mp_timing=mp_timingI;
}

//sets the music for the channels
void mp_setChannels(int *channel1I, int *channel2I, int *channel3I, int *channel4I, int *channel5I)
{
	mp_channel1 = channel1I;
	mp_channel2 = channel2I;
	mp_channel3 = channel3I;
	mp_channel4 = channel4I;
	mp_channel5 = channel5I;
}

//sets the active channels
void mp_setActiveChannels(u8 channel1AI, u8 channel2AI, u8 channel3AI, u8 channel4AI, u8 channel5AI)
{
	mp_channel1Active = channel1AI;
	mp_channel2Active = channel2AI;
	mp_channel3Active = channel3AI;
	mp_channel4Active = channel4AI;
	mp_channel5Active = channel5AI;
}

//loads a song from the library
void mp_loadSong(int index)
{
	mp_setChannels(ml_music_c1[index], ml_music_c2[index], ml_music_c3[index], ml_music_c4[index], ml_music_c5[index]);
	mp_setActiveChannels(ml_music_ca1[index], ml_music_ca2[index], ml_music_ca3[index], ml_music_ca4[index], ml_music_ca5[index]);
	mp_setNoteCount(ml_music_nc[index]);
	mp_setTiming(ml_music_t[index]);
	mp_setReplay(ml_music_r[index]);
}

//plays a single note (thy to DanB)
void mp_playNote(u8 chan, u16 freq, u8 vol, u8 fade) {
    SND_REGS[chan].SxINT = 0; 
	
	//Set up the frequency the channel will play
    SND_REGS[chan].SxFQH = freq >> 8;
    SND_REGS[chan].SxFQL = freq & 0xFF;

    //Set the volume for the channel (0-15, same for left/right)
    SND_REGS[chan].SxLRV = (vol << 4) | vol;

 	if(freq==PAU)
	{
		SND_REGS[chan].SxINT = 0; 
		return;
	}
	
	if(!fade)
	{
		SND_REGS[chan].SxEV0 = 0xF0;
		SND_REGS[chan].SxEV1 = 0x00;
	}
	else
	{
		SND_REGS[chan].SxEV0 = 0xF7;
		SND_REGS[chan].SxEV1 = 0x01;
	}

    //Start the channel without interval
    SND_REGS[chan].SxINT = 0x9F;
}

//starts the music player
void mp_playerStart()
{
	mp_currNote=0;
	mp_songTick=0;
	mp_play=1;
	
}

//stops the music player
void mp_playerStop()
{
	mp_currNote=0;
	mp_songTick=0;
	mp_play=0;
	mp_playNote(0, PAU, 8, 0);
	mp_playNote(1, PAU, 8, 0);
	mp_playNote(2, PAU, 8, 0);
	mp_playNote(3, PAU, 8, 0);
	mp_playNote(4, PAU, 8, 0);
	
}


//this function should be called in every frame to play music
//It could be more generic, but I was lazy
int mp_playMusic() {

	if(mp_play)
	{
		if(mp_currNote==0) //play first note of song
		{
			mp_targetTick=mp_timing[0];
			
			if(mp_channel1Active) {
				mp_playNote(0, mp_channel1[mp_currNote], 8, 0);
			}
			if(mp_channel2Active) {
				mp_playNote(1, mp_channel2[mp_currNote], 8, 0);
			}
			if(mp_channel3Active) {
				mp_playNote(2, mp_channel3[mp_currNote], 8, 0);
			}
			if(mp_channel4Active) {
				mp_playNote(3, mp_channel4[mp_currNote], 8, 0);
			}
			if(mp_channel5Active) {
				mp_playNote(4, mp_channel5[mp_currNote], 8, 0);
			}
		}

		if (mp_songTick >= mp_targetTick) { // it is timeto play a new note
		
			mp_currNote++;
			mp_songTick=0;
			
			if (mp_currNote >= mp_noteCount) //song ends, if replay active we will start over again
			{
				//mute all active channels
				if(mp_channel1Active) {
					mp_playNote(0, PAU, 8, 0);
				}
				if(mp_channel2Active) {
					mp_playNote(1, PAU, 8, 0);
				}
				if(mp_channel3Active) {
					mp_playNote(2, PAU, 8, 0);
				}
				if(mp_channel4Active) {
					mp_playNote(3, PAU, 8, 0);
				}
				if(mp_channel5Active) {
					mp_playNote(4, PAU, 8, 0);
				}
				
				//start replay if neccessary
				if(mp_replay==0)
				{
					mp_playerStop();
				}
				else if(mp_replay > 0)
				{
					mp_currNote=0;
					mp_songTick=0;
					mp_replay--;
				}
				else if(mp_replay < 0)
				{
					mp_currNote=0;
					mp_songTick=0;
				}
				return mp_currNote;
				
			}
			else {
			
				//we have to play a new note
				mp_targetTick = mp_timing[mp_currNote];
				
				if(mp_channel1Active) {
					mp_playNote(0, mp_channel1[mp_currNote], 8, 0);
				}
				if(mp_channel2Active) {
					mp_playNote(1, mp_channel2[mp_currNote], 8, 0);
				}
				if(mp_channel3Active) {
					mp_playNote(2, mp_channel3[mp_currNote], 8, 0);
				}
				if(mp_channel4Active) {
					mp_playNote(3, mp_channel4[mp_currNote], 8, 0);
				}
				if(mp_channel5Active) {
					mp_playNote(4, mp_channel5[mp_currNote], 8, 0);
				}
			}
		}
		else if((mp_songTick + 1) == mp_targetTick ) // the last tick of a note fades, makes the song more smoth (or maybe changes nothing)
		{
			if(mp_channel1Active) {
				mp_playNote(0, mp_channel1[mp_currNote], 8, 1);
			}
			if(mp_channel2Active) {
				mp_playNote(1, mp_channel2[mp_currNote], 8, 1);
			}
			if(mp_channel3Active) {
				mp_playNote(2, mp_channel3[mp_currNote], 8, 1);
			}
			if(mp_channel4Active) {
				mp_playNote(3, mp_channel4[mp_currNote], 8, 1);
			}
			if(mp_channel5Active) {
				mp_playNote(4, mp_channel5[mp_currNote], 8, 1);
			}
		}
			
		mp_songTick++;
	}
	
	return mp_currNote;
}
